import React, { useState } from 'react';
import ReactDOM from 'react-dom';
import ReactDropdown from 'react-dropdown';
import { scaleLinear, extent, format } from 'd3';

import { useData } from './useData';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Marks';
import { Dropdown } from './Dropdown';

const width = 960;
const height = 450;
const margin = { top: 5, right: 30, bottom: 85, left: 90 };
const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;

// dropdown options
const options = [
  { value: 'sepal_length', label: 'Sepal Length' },
  { value: 'sepal_width', label: 'Sepal Width' },
  { value: 'petal_length', label: 'Petal Length' },
  { value: 'petal_width', label: 'Petal Width' },
];

const getLabel = (val) => options.filter((x) => x.value === val)[0].label;

const App = () => {
  // state
  const data = useData();
  const initialXAttr = 'petal_length';
  const initialYAttr = 'sepal_width';
  const [xAttr, setXAttr] = useState(initialXAttr);
  const [yAttr, setYAttr] = useState(initialYAttr);

  if (!data) {
    return <pre>Loading...</pre>;
  }

  // x,y attributes and value accessors
  const xValue = (d) => d[xAttr];
  const xAxisLabel = getLabel(xAttr);
  const xAxisLabelOffset = 50;

  const yValue = (d) => d[yAttr];
  const yAxisLabel = getLabel(yAttr);
  const yAxisLabelOffset = 40;

  // tick format
  const siFormat = format('.2s');
  const xAxisTickFormat = (tickValue) => siFormat(tickValue).replace('G', 'B');

  // scales
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight])
    .nice();

  return (
    <>
      <div class="menus">
        <span className="dropdown-label">X</span>
        <ReactDropdown
          options={options}
          id={'x-select'}
          value={xAttr}
          onChange={({ value }) => setXAttr(value)}
        />

        <span className="dropdown-label">Y</span>
        <ReactDropdown
          options={options}
          id={'y-select'}
          value={yAttr}
          onChange={({ value }) => setYAttr(value)}
        />
      </div>

      <svg width={width} height={height}>
        <g transform={`translate(${margin.left},${margin.top})`}>
          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            tickFormat={xAxisTickFormat}
            tickOffset={10}
          />

          <AxisLeft yScale={yScale} innerWidth={innerWidth} tickOffset={10} />

          <text
            className="axis-label"
            x={innerWidth / 2}
            y={innerHeight + xAxisLabelOffset}
            textAnchor="middle"
          >
            {xAxisLabel}
          </text>

          <text
            className="axis-label"
            textAnchor="middle"
            transform={`translate(${-yAxisLabelOffset},${
              innerHeight / 2
            }) rotate(-90)`}
          >
            {yAxisLabel}
          </text>

          <Marks
            data={data}
            xScale={xScale}
            yScale={yScale}
            xValue={xValue}
            yValue={yValue}
            tooltipFormat={xAxisTickFormat}
          />
        </g>
      </svg>
    </>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
